import java.awt.Graphics;
import java.awt.Point;

/**
 * This class implements <font color="blue">The Wild Blue Creature</font>. Here
 * is it: <img src="gif/ball0.gif" alt="portrait of the WBC" height="16"
 * width="16">
 */
public class Ball extends Movable {
	/**
	 * This is speed of light for the creature. It cannot move faster.
	 */
	final static int MAX_SPEED = Movable.radius - 2;
	private int ndx, ndy;
	private int imageno = 0;

	/**
	 * Create a new creature.
	 * 
	 * @param startx
	 *            Initial x coordinate.
	 * @param starty
	 *            Initial y coordinate.
	 * @param speedx
	 *            Initial horizontal velocity.
	 * @param speedy
	 *            Initial vertical velocity.
	 * @param _map
	 *            Reference to the level map. <br>
	 *            Note that velocities cannot be both 0 (it <i>is</i> a legal
	 *            value, but it is changed immediately in the constructor).
	 */
	public Ball(int startx, int starty, int speedx, int speedy, Map _map) {
		super(startx, starty, speedx, speedy, _map);
		if (dx == 0 && dy == 0)
			dx = dy = Map.COEFF;
	}

	/**
	 * Upadte the object's picture.
	 */
	public void update(Graphics g, Xonix parent) {
		if (!dirty)
			return;
		map.updateRect(g, last.x - radius, last.y - radius, last.x + radius,
				last.y + radius, parent);
		g.drawImage(parent.ballImages[imageno++], center.x - radius, center.y
				- radius, radius * 2, radius * 2, parent);
		imageno %= 8;
		remember_last();
		dirty = false;
	}

	/**
	 * Moves the creature.
	 */
	public void move() {
		super.move();
		Point collp = new Point(0, 0);
		if (map.find(this, collp, Map.COLLISION)) {
			int dir = map.wallDirection(collp);
			if ((dir & Map.WALL_VERTICAL) != 0)
				dx = -dx;
			if ((dir & Map.WALL_HORIZONTAL) != 0)
				dy = -dy;
		}
		// /// int x=get_center().x,y=get_center().y;
		// if(x<map.dWall||y<3*map.dWall||
		// x>=map.maxx-map.dWall||y>=map.maxy-map.dWall)
		// {
		// x=(int)(Math.random()*(map.maxx-map.dWall*4)+map.dWall*2);
		// y=(int)(Math.random()*(map.maxy-map.dWall*4)+map.dWall*2);
		// super.move(x,y);
		// }
		dirty = true;
		ndx = dx;
		ndy = dy;
	}

	/**
	 * Draw the creature on the parent
	 */
	public void paint(Graphics g, Xonix parent) {
		dirty = true;
		update(g, parent);
	}

	/**
	 * Check collision with another creature. If collides, bounce.
	 * 
	 * @param other
	 *            Object to check.
	 */
	public void check_collision(Ball other) {
		if (collides(other)) {
			// collision is absolutely hard, without energy loss.
			// balls just swap their velocities.
			ndx = other.dx;
			ndy = other.dy;
		}
	}

	/**
	 * Check collision with a spider. If collides, bounce with acceleration.
	 * 
	 * @param other
	 *            The spider.
	 */
	public void check_collision(Spider other) {
		if (collides(other)) {
			ndx = other.getdx() + dx;
			ndy = other.getdy() + dy;
			if (ndx == 0 && other.getdx() != 0)
				ndx = other.getdx() + Map.COEFF
						* (other.getdx() / Math.abs(other.getdx()));
			if (ndy == 0 && other.getdy() != 0)
				ndy = other.getdy() + Map.COEFF
						* (other.getdy() / Math.abs(other.getdy()));
		}
	}

	/**
	 * Makes consequences of the collisions to be effective.
	 */
	public void commit_collisions() {
		// speed is limited by the MAX_SPEED - spped of light here.
		if (ndx > MAX_SPEED)
			ndx = MAX_SPEED;
		if (ndx < -MAX_SPEED)
			ndx = -MAX_SPEED;
		if (ndy > MAX_SPEED)
			ndy = MAX_SPEED;
		if (ndy < -MAX_SPEED)
			ndy = -MAX_SPEED;

		dx = ndx;
		dy = ndy;
	}
}
