import java.awt.*;
import java.applet.Applet;
import java.util.Vector;
import java.net.*;

/**
 * The main class. Manages the game, creates and moves the parts.
 */
public class Xonix extends Applet implements Runnable {
  private Ball[] balls;
  private Thread ticker;
  private Spider spider;
  private Map map;
      
  private int level;
  private boolean dirty;
  private boolean locked;
  private boolean running=false;
  // controls the live increment for large score
  // doesn't really conts hundreds, rather hundreds of hundreds
  private int hundreds=0;
    
  private static int XSIZE = 400;
  private static int YSIZE = 400;
  private static int BALLS_ON_START=2;
  private static int SPIDER_SIZE=Movable.radius+Map.COEFF;
  private static int START_LIVES=4;
  private static int MAX_PERCENT=70;
  private static int LIVES_INC_BARRIER=10000;
 private int lives=START_LIVES;
  private static int SLEEP_TIME=10;
  /**
   * Images for the Ball class.
   * @see Ball
   */
  public static Image[] ballImages=null;
  /**
   * Images for the Spider class.
   * @see Spider
   */
  public static Image[] spiderImages=null;
  /**
   * Percents of the space captured.
   */
  public int percent=0;
  /**
   * Current score count.
   */
  public int score=0;
  /**
   * Initial method.
   * @return Funny game:)
   */
  public void init() {
    resize(XSIZE,YSIZE);
    ticker=null;
    restart();
    setBackground(Color.black);
    URL[] ballURLs=new URL[8];
    URL[] spiderURLs=new URL[2];
    ballImages=new Image[8];
    spiderImages=new Image[2];
    setFont(new Font("Dialog",0,10));
    try
	{
	  for(int i=0;i<ballURLs.length;i++)
	    ballURLs[i]=new URL(getDocumentBase(),"gif/ball"+i+".gif");
	  spiderURLs[0]=new URL(getDocumentBase(),"gif/spider.gif");
	  spiderURLs[1]=new URL(getDocumentBase(),"gif/spider_out.gif");
	  for(int i=0;i<ballURLs.length;++i)
	    ballImages[i]=getImage(ballURLs[i]);
	  for(int i=0;i<spiderURLs.length;++i)
	    spiderImages[i]=getImage(spiderURLs[i]);
	}
    catch(MalformedURLException e)
	{
	  System.out.println(e);
	}
    dirty=true;
  }

  /**
   * Init game level.
   * @param lev Level to be.
   */
  private void init_game(int lev)
      {
	level=lev;
	if(level!=0) score+=800+200*level;
	check_hundreds();
	map = new Map(XSIZE,YSIZE,SPIDER_SIZE);
	System.out.println("map done");
	balls = new Ball[level+BALLS_ON_START];
	for(int i=0;i<level+BALLS_ON_START;i++)
	  balls[i]=new Ball((int)(Math.random()*(XSIZE-SPIDER_SIZE*4)+
				      SPIDER_SIZE*2),
				    (int)(Math.random()*(YSIZE-SPIDER_SIZE*4)+
					  SPIDER_SIZE*2),
				    (int)(Math.random()*2*Ball.MAX_SPEED)-
			    Ball.MAX_SPEED,
				    (int)(Math.random()*2*Ball.MAX_SPEED)-
			    Ball.MAX_SPEED,
				    map);
	System.out.println("balls done");
	spider=new Spider(100,SPIDER_SIZE*3,map);
	System.out.println("spider done");
	System.gc();
	System.out.println("gc done");
	dirty=true;
	repaint();
      }
  /**
   * Applet starting.
   */
  public void start()
      {
	if(ticker == null)
	  ticker=new Thread(this);
	ticker.start();
      }
  /**
   * Applet stopping.
   */
  public void stop()
      {
	ticker.stop();
	ticker=null;
      }

  private void spider_died()
      {
	if(lives>0) lives--;
	System.out.println("dead");
	map.wipe_path();
	spider=new Spider(100,SPIDER_SIZE*3,map);
	dirty=true;
	repaint();
      }

  /**
   * Applet paining.
   */
  public void paint(Graphics g) {
    if(locked||running) return;
    locked=true;
    map.paint(g,this);
    spider.paint(g,this);
    for(int i=0;i<balls.length;++i)
      balls[i].paint(g,this);
    locked=false;
 }
  
  /**
   * Applet updating.
   */
  public void update(Graphics g) 
    {
        if(locked||running) return;
	locked=true;
	  map.update(g,this);
	spider.update(g,this);
	for(int i=0;i<balls.length;++i)
	  balls[i].update(g,this);
	if(lives==0)
	  g.drawString("Game over",XSIZE/2,YSIZE/2);
	int x=2*SPIDER_SIZE;
	for(int i=0;i<lives;i++)
	    {
	      g.drawImage(spiderImages[0],x-Movable.radius,
		      SPIDER_SIZE-Movable.radius,
			  Movable.radius*2+1,Movable.radius*2+1,this);
	      x+=2*SPIDER_SIZE;
	    }
	g.setColor(Color.white);
	g.drawString("level: "+level+" captured: "+percent+"% score: "+score,
		     x,SPIDER_SIZE*2);
	dirty=false;
	locked=false;
      }

  /**
   * The main method - does all The Job<sup><TT>TM</TT></sup>.
   */
  public void run()
  {
    while(ticker != null)
	{
	      running=true;
	       if(lives==0)
		  {
		    // no lives left - die.die.die
		    // here only restart will do
		    try {
		      running=false;
		      Thread.sleep(500);
		    } catch (InterruptedException e) {}
		    running=true;
		    continue;
		  }
	      repaint();
	      for(int i=0;i<balls.length;++i)
		balls[i].move();
	      for(int i=0;i<balls.length;++i)
		{
		  for(int j=0;j<balls.length;j++)
		    if(i!=j)
		      balls[i].check_collision(balls[j]);
		  balls[i].check_collision(spider);
		}
	      spider.move();
	      for(int i=0;i<balls.length;++i)
		  {
		    balls[i].commit_collisions();
		    if(spider.check_collision(balls[i]))
			{
			  spider_died();
			  break;
			}
		  }
	      if(spider.has_path())
		  {
		    int oldfilled=map.filled();
		    map.fill_all();
		    for(int i=0;i<balls.length;++i)
		      map.flood_fill(balls[i].get_center());
		    dirty=true;
		    percent=map.percent();
		    score+=(map.filled()-oldfilled)/10;
		    check_hundreds();
		    System.out.println("percent is "+percent);
		    if(map.percent()>=MAX_PERCENT)
			{
			  level++;
			  init_game(level);
			}
		    spider.clear_path();
		    repaint();
		    System.out.println("done with path");
		  }
	      
	      locked=false;
	      running=false;
	      do{
		try {
		  Thread.sleep(SLEEP_TIME);
		} catch (InterruptedException e) {}
	      }
	      while(locked);
			      
	    }
	}
  
  /**
   * This is called when Joe User presses a key.
   */
  public boolean keyDown(Event e,int key)
      {
	switch(key)
	    {
	    case 'R':
	    case 'r':
	      restart();
	      break;
	    case 'I':
	    case 'i':
	      spider.up();
	      break;
	    case 'K':
	    case 'k':
	      spider.down();
	      break;
	    case 'j':
	    case 'J':
	      spider.left();
	      break;
	    case 'L':
	    case 'l':
	      spider.right();
	      break;
	    case ' ':
	      spider.freeze();
	      break;
	    case 'p':
	      System.out.println(spider.get_center());
	      System.out.println(map.position(spider.get_center()));
	      break;
	    }
	return true;
      }

  // check if it isn't the time to add a life
  private void check_hundreds()
  {
    if(score/LIVES_INC_BARRIER != hundreds)
	{
	  hundreds=score/LIVES_INC_BARRIER;
	  lives++;
	  dirty=true;
	  repaint();
	}
    
  }
    
  // restart the game
  private void restart()
      {
	score=hundreds=0;
	lives=START_LIVES;
	init_game(0);
      }
  
}
